﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using WWWings35_GO.EF;
using Perseus;

namespace WWWings35_DZ.EF
{

 /// <summary>
 /// Datenmanager-Klasse für Flüge
 /// Verwendet ADO.NET EF
 /// </summary>
 public class FlugDataManager : DataManagerBase
 {
   WWWings6EF db = new WWWings6EF();

  #region Hilfsroutinen
  // Interne Hilfsroutine: KeySelector für Sortierung
  private IQueryable<Flug> AddSortingAndPagingToQuery(string Sortierung, IQueryable<Flug> Query, int Start, int Anzahl)
  {
   string SortierungFeldName = Sortierung.Replace(" DESC", "");
   if (SortierungFeldName == "") SortierungFeldName = "Abflugort";
   Expression<Func<Flug, string>> KeySelector = GetKeyStringSelector(SortierungFeldName);

   // Unterscheiden zwischen Sortierrichtung
   if (Sortierung.Contains("DESC"))
   {
    Query = Query.OrderByDescending(KeySelector);
   }
   else
   {
    Query = Query.OrderBy(KeySelector);
   }

   // Blättern und Konvertieren
   Query = Query.Skip(Start).Take(Anzahl);
   return Query;
  }
  #endregion

  #region Count
  /// <summary>
  /// Anzahl der Flüge
  /// </summary>
  /// <returns></returns>
  public long CountFluege()
  {
   return (from f in db.Flug select f).Count();
  }

  /// <summary>
  /// Anzahl der Flüge
  /// </summary>
  /// <returns></returns>
  public long CountFluegeVonOrt(string Ort)
  {
   return (from f in db.Flug where f.Abflugort == Ort select f).Count();
  }

  /// <summary>
  /// Anzahl der Flüge auf einer Route
  /// </summary>
  public long CountFluegeRoute(string Abflugort, string Zielort)
  {
   return (from f in db.Flug where f.Abflugort == Abflugort && f.Zielort == Zielort select f).Count();
  }
  #endregion

  #region Einzelflug
  /// <summary>
  /// Holt einen Flug nach Primärschlüssel. Inkludiert alle Passagieren und deren Details sowie den Pilot
  /// </summary
  public Flug GetFlug(long Nr)
  {
   // Flug flug = (from f in db.Flugs where f.FlugNr == Nr select f).SingleOrDefault();

   // für Unit Test Demos: Single() statt SingleOrDefault()
   Flug flug = (from f in db.Flug.Include("Passagier.Person").Include("Pilot.Mitarbeiter.Person") where f.FlugNr == Nr select f).FirstOrDefault();
   return flug;
  }

  /// <summary>
  /// Holt einen Flug nach Primärschlüssel. Verwendet EntityBag Codeplex Projekt!
  /// </summary>
  public EntityBag<Flug> GetFlug_AsEntityBag(long Nr)
  {
   // Flug flug = (from f in db.Flugs where f.FlugNr == Nr select f).SingleOrDefault();

   // für Unit Test Demos: Single() statt SingleOrDefault()
   Flug flug = (from f in db.Flug.Include("Passagier").Include("Passagier.Person").Include("Pilot.Mitarbeiter.Person") where f.FlugNr == Nr select f).FirstOrDefault();

   EntityBag<Flug> FlugBag = db.CreateEntityBag<Flug>(flug);

   return FlugBag;
  }
  #endregion

  #region Flugmenge
  /// <summary>
  /// Holt Flüge von einem Abflugort
  /// </summary>
  public List<Flug> GetFluegeVonOrt(string Ort)
  {
   List<Flug> fluege = (from f in db.Flug where f.Abflugort == Ort select f).ToList();
   return fluege;
  }

  /// <summary>
  /// Holt Flüge auf einer Route
  /// </summary>
  public List<Flug> GetFluegeRoute(string Abflugort, string Zielort)
  {
   List<Flug> fluege = (from f in db.Flug where f.Abflugort == Abflugort && f.Zielort == Zielort select f).ToList();
   return fluege;
  }

  /// <summary>
  /// Holt Flüge  in einem Zeiraum
  /// </summary>
  public List<Flug> GetFluegeZeitraum(DateTime ZeitraumStart, DateTime ZeitraumEnde)
  {
   return (from flug in db.Flug where flug.Datum >= ZeitraumStart && flug.Datum <= ZeitraumEnde select flug).ToList();
  }


  /// <summary>
  /// Holt Flüge auf einer Route in einem Zeiraum
  /// </summary>
  public List<Flug> GetFluegeRouteZeitraum(string Abflugort, string Zielort, DateTime ZeitraumStart, DateTime ZeitraumEnde)
  {
   return (from flug in db.Flug where flug.Abflugort == Abflugort && flug.Zielort == Zielort && flug.Datum >= ZeitraumStart && flug.Datum <= ZeitraumEnde select flug).ToList();
  }

  /// <summary>
  /// Hollt alle Flüge
  /// </summary>
  /// <returns></returns>
  public List<Flug> GetFluege()
  {
   List<Flug> fluege = (from f in db.Flug select f).ToList();
   return fluege;
  }

  /// <summary>
  /// Holt alle Flüge mit Sortieren und Blättern
  /// </summary>
  public List<Flug> GetFluege_Paging(string Sortierung, int Anzahl, int Start)
  {
   // Basisabfrage nur mit Bedingung
   var Query = (from f in db.Flug select f);
   // Auswerten der Parameter
   Query = AddSortingAndPagingToQuery(Sortierung, Query, Start, Anzahl);
   // Konvertieren
   List<Flug> fluege = Query.ToList();
   return fluege;
  }

  /// <summary>
  /// Holt Flüge mit Ortsfilter und Sortieren/Blättern
  /// </summary>
  public List<Flug> GetFluegeVonOrt_Paging(string Ort, string Sortierung, int Anzahl, int Start)
  {
   // Basisabfrage nur mit Bedingung
   var Query = (from f in db.Flug where f.Abflugort == Ort select f);
   // Auswerten der Parameter
   Query = AddSortingAndPagingToQuery(Sortierung, Query, Start, Anzahl);
   // Konvertieren
   List<Flug> fluege = Query.ToList();
   return fluege;
  }

  /// <summary>
  /// Holt Flüge mit Routenfilter und Sortieren/Blättern
  /// </summary>
  public List<Flug> GetFluegeRoute_Paging(string Abflugort, string Zielort, string Sortierung, int Anzahl, int Start)
  {
   // Basisabfrage nur mit Bedingung
   var Query = (from f in db.Flug where f.Abflugort == Abflugort && f.Zielort == Zielort select f);
   // Auswerten der Parameter
   Query = AddSortingAndPagingToQuery(Sortierung, Query, Start, Anzahl);
   // Konvertieren
   List<Flug> fluege = Query.ToList();
   return fluege;
  }
  #endregion

  #region Flüge ändern
  /// <summary>
  /// Speichern eines geänderten Flugs
  /// </summary>
  /// <param name="FlugNeu"></param>
  /// <returns></returns>
  public bool SaveFlug(Flug FlugNeu)
  {
   UpdateOrAddFlug(FlugNeu);
   db.SaveChanges();
   return true;
  }


  /// <summary>
  /// Notwendig für WCF
  /// </summary>
  /// <param name="FlugNeu"></param>
  private void UpdateOrAddFlug(Flug FlugNeu)
  {
   object FlugAlt;
   System.Data.EntityKey key;
   if (FlugNeu.EntityState != EntityState.Detached) return; // keine Sonderbehandlung notwendig!

   // Hole EF-Schlüssel für das Objekt
   if (FlugNeu.EntityKey == null)
   {
    key = db.CreateEntityKey("Flug", FlugNeu);
    Console.WriteLine("Saving Flug " + FlugNeu.FlugNr + " State: " + FlugNeu.EntityState.ToString() + " Key Neu: " + key.EntityKeyValues[0]);
   }
   else
   {
    key = FlugNeu.EntityKey;
    Console.WriteLine("Saving Flug " + FlugNeu.FlugNr + " State: " + FlugNeu.EntityState.ToString() + " Key: " + key.EntityKeyValues[0]);
   }

   // Hole Originalobjekt
   if (db.TryGetObjectByKey(key, out FlugAlt))
   {
    // "Wiedereinspielen" der Änderungen
    db.ApplyPropertyChanges(key.EntitySetName, FlugNeu);
    Console.WriteLine("  Saving existing Object");
   }
   else
   {
    // Neues Objekt
    db.AddToFlug(FlugNeu);
    Console.WriteLine("  Saving new Object");
   }
  }

  /// <summary>
  /// Speichern aller Änderungen im Kontext
  /// </summary>
  public string SaveFluege()
  {

   // Optional für Anzeige der Anzahl geänderter Daten
   string Rueckgabe = "";
   Rueckgabe += "Geändert: " + db.ObjectStateManager.GetObjectStateEntries(System.Data.EntityState.Modified).Count();
   Rueckgabe += " Neu: " + db.ObjectStateManager.GetObjectStateEntries(System.Data.EntityState.Added).Count();
   Rueckgabe += " Gelöscht: " + db.ObjectStateManager.GetObjectStateEntries(System.Data.EntityState.Deleted).Count();
   db.SaveChanges();
   return Rueckgabe;
  }

  /// <summary>
  /// Speichern aller geänderten und hinzugefügten Flüge
  /// </summary>
  public string SaveFluege(List<Flug> fluege)
  {
   foreach (Flug f in fluege)
   {
    UpdateOrAddFlug(f);
   }
   return SaveFluege();
  }

  /// <summary>
  /// Aufruf, wenn in UI eine Zeile ergänzt wurde
  /// </summary>
  public bool MarkFlugForAdding(Flug flug)
  {
   db.AddToFlug(flug);
   return true;
  }

  /// <summary>
  /// Für Desktop-UI: Aufruf, wenn in UI eine Zeile gelöscht wurde
  /// </summary>
  public bool MarkFlugForDeletion(Flug flug)
  {
   // nur für WCF-Dienste
   if (flug.EntityState == EntityState.Detached)
    db.Attach(flug);

   db.DeleteObject(flug);
   return true;
  }
  #endregion

 }
}
