﻿// Demo von Microsoft. Angepasst von HS.

using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Data.Entity.Design;
using System.Data.Metadata.Edm;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Xml;
using System.Xml.Linq;

namespace de.WWWings
{
 /// <summary>
 /// This is the generator class. 
 /// When the 'Custom Tool' property of a .edmx file in a C# or VB project item is set to "WWWingsEFCodeGenerator", 
 /// Visual Studio will call the GenerateCode function which will return the contents of the generated file back
 /// to the project system
 /// </summary>
 [ComVisible(true)]
 [Guid("322EB3BE-19FB-4b6d-9370-1A6474C60D00")]
 // old: WWWingsEFCodeGenerator
 public class WWWingsEFCodeGenerator : BaseCodeGeneratorWithSite
 {
  /// <summary>
  /// Make the generated (a.k.a. code-behind) file have the file extension ".Designer.cs" or ".Designer.vb"
  /// </summary>
  /// <returns></returns>
  protected override string GetDefaultExtension()
  {
   return (".Designer" + base.GetDefaultExtension());
  }

  /// <summary>
  /// Override to generate and return code from the contents of the input file contents
  /// Does the following:
  /// - extracts CSDL content from the input EDMX content
  /// - calls public code generation APIs in System.Data.Entity.Design with the appropriate language
  /// - hooks code generation events to add custom CLR attributes to the gerenated code
  /// - (the attributes to add are present as CSDL annotations in the EDMX file)
  /// - uses EDM metadata APIs to find CSDL annotations in the EDMX file and turns them into CLR attributes
  /// - logs warnings & errors to the Visual Studio error window
  /// - returns the generated code as a byte array to Visual Studio
  /// </summary>
  /// <param name="inputFileContent">Content of the input EDMX file</param>
  /// <returns>Generated source code (C# or VB) as a byte array</returns>
  protected override byte[] GenerateCode(string inputFileContent)
  {
   byte[] generatedCodeAsBytes = null;

   try
   {
    // Check for and get CSDL content
    XElement csdlContent = ExtractCsdlContent(inputFileContent);
    if (csdlContent == null)
    {
     throw new InvalidOperationException("No CSDL content in input file");
    }

    // Reader over CSDL content within EDMX
    using (XmlReader csdlReader = csdlContent.CreateReader())
    {
     using (StringWriter codeWriter = new StringWriter(CultureInfo.InvariantCulture))
     {
      // Determine language for generated code
      LanguageOption languageOption = LanguageOption.GenerateCSharpCode;
      string fileExtension = base.GetCodeProvider().FileExtension;
      if (fileExtension != null && fileExtension.Length > 0)
      {
       fileExtension = "." + fileExtension.TrimStart(".".ToCharArray());
      }
      if (fileExtension.EndsWith(".vb", StringComparison.InvariantCultureIgnoreCase))
      {
       languageOption = LanguageOption.GenerateVBCode;
      }
      else if (fileExtension.EndsWith(".cs", StringComparison.InvariantCultureIgnoreCase))
      {
       languageOption = LanguageOption.GenerateCSharpCode;
      }
      else
      {
       throw new InvalidOperationException("Unsupported project language. Only C# and VB are supported.");
      }

      if (base.CodeGeneratorProgress != null)
      {
       // Report that we are 33% done
       base.CodeGeneratorProgress.Progress(33, 100);
      }

      // Create code generator and wire up events
      EntityClassGenerator classGenerator = new EntityClassGenerator(languageOption);
      classGenerator.OnTypeGenerated += new TypeGeneratedEventHandler(OnTypeGenerated);
      classGenerator.OnPropertyGenerated += new PropertyGeneratedEventHandler(OnPropertyGenerated);
      
      if (base.CodeGeneratorProgress != null)
      {
       // Report that we are 66% done
       base.CodeGeneratorProgress.Progress(66, 100);
      }

      // Generate code and log warnings/errors to the Visual Studio error window
      IList<EdmSchemaError> errors = classGenerator.GenerateCode(csdlReader, codeWriter);
      if (errors != null)
      {
       foreach (EdmSchemaError error in errors)
       {
        int line = (error.Line == 0) ? 0 : (error.Line - 1);
        int column = (error.Column == 0) ? 0 : (error.Column - 1);

        if (error.Severity == EdmSchemaErrorSeverity.Warning)
        {
         base.GeneratorWarning(0, error.Message, (uint)line, (uint)column);
        }
        else
        {
         base.GeneratorError(4, error.Message, (uint)line, (uint)column);
        }
       }
      }

      // Erweitern durch Code im Klartext am Ende!!!
      codeWriter.WriteLine("// ==== Ende des Standardcodes. Hier kommt eigener Code:");
      codeWriter.WriteLine("class MyObjectContext : global::System.Data.Objects.ObjectContext {  } ");

      // Convert generated code into bytes to return to Visual Studio
      generatedCodeAsBytes = Encoding.UTF8.GetBytes(codeWriter.ToString());

      if (base.CodeGeneratorProgress != null)
      {
       // Report that we are 100% done
       base.CodeGeneratorProgress.Progress(100, 100);
      }
     }
    }
   }
   catch (Exception e)
   {
    base.GeneratorError(4, e.Message, 1, 1);

    // Returning null signifies that generation has failed
    generatedCodeAsBytes = null;
   }

   return generatedCodeAsBytes;
  }

  public void Log(string s)
  {
   System.IO.StreamWriter sw = new StreamWriter(@"c:\vslog.txt", true);
   sw.WriteLine(s);
   sw.Close();
  }

  /// <summary>
  /// This event is raised by code generation after a type is generated
  /// </summary>
  /// <param name="sender"></param>
  /// <param name="eventArgs"></param>
  private void OnTypeGenerated(object sender, TypeGeneratedEventArgs eventArgs)
  
  {
   // HS: Protokollierung in Textdatei und Erzeugen Member GetKey() für alle Entity-Klassen (nicht aber für den Kontext!)

   string TypeName = eventArgs.TypeSource.MetadataProperties["Name"].Value.ToString();
   // nicht für den Kontext
   if (TypeName.Contains("Entities")) return;

   // Protokollierung
   Log("OnTypeGenerated: " + sender.GetType().FullName);

   foreach (MetadataProperty p in eventArgs.TypeSource.MetadataProperties)
   {
    Log(p.Name + " = " + p.Value);
   }

   // Neues Mitglied GetKey() erzeugen    
   CodeMemberMethod GetKey = new CodeMemberMethod();
   GetKey.Name = "GetKey";
   GetKey.ReturnType = new CodeTypeReference("System.Object");
   CodeStatement s = new CodeStatement();
   GetKey.Statements.Add( new System.CodeDom.CodeCommentStatement("Hier ein Beispiel für die Anpassung des Codegenerators!"));
   // erzeugt: return(this.flug.EntityKey.EntityKeyValues[0].Value)
   CodeStatement statement = new CodeMethodReturnStatement(
new CodePropertyReferenceExpression(
new CodeIndexerExpression(
new CodePropertyReferenceExpression(
new CodePropertyReferenceExpression(new CodeThisReferenceExpression(), "EntityKey"),
"EntityKeyValues"), new CodePrimitiveExpression(0)), "Value"));
   GetKey.Statements.Add(statement);
   eventArgs.AdditionalMembers.Add(GetKey);
  }

  /// <summary>
  /// This event is raised by code generation after a property is generated
  /// </summary>
  /// <param name="sender"></param>
  /// <param name="eventArgs"></param>
  private void OnPropertyGenerated(object sender, PropertyGeneratedEventArgs eventArgs)
  {
   Log("OnPropertyGenerated: " + sender.GetType().FullName);
  }

 

  /// <summary>
  /// Extracts CSDL content from the EDMX file content and returns it as an XElement
  /// </summary>
  /// <param name="inputFileContent">EDMX file conmtent</param>
  /// <returns>CSDL content XElement</returns>
  private XElement ExtractCsdlContent(string inputFileContent)
  {
   XElement csdlContent = null;
   XNamespace edmxns = "http://schemas.microsoft.com/ado/2007/06/edmx";
   XNamespace edmns = "http://schemas.microsoft.com/ado/2006/04/edm";

   XDocument edmxDoc = XDocument.Load(new StringReader(inputFileContent));
   if (edmxDoc != null)
   {
    XElement edmxNode = edmxDoc.Element(edmxns + "Edmx");
    if (edmxNode != null)
    {
     XElement runtimeNode = edmxNode.Element(edmxns + "Runtime");
     if (runtimeNode != null)
     {
      XElement conceptualModelsNode = runtimeNode.Element(edmxns + "ConceptualModels");
      if (conceptualModelsNode != null)
      {
       csdlContent = conceptualModelsNode.Element(edmns + "Schema");
      }
     }
    }
   }
   return csdlContent;
  }
 }
}